package com.lodgon.dali.core.oauth;

import com.lodgon.dali.core.oauth.ejb.OAuthBean;
import com.lodgon.dali.core.oauth.entity.DaliToken;
import com.sun.jersey.oauth.server.spi.OAuthConsumer;
import com.sun.jersey.oauth.server.spi.OAuthProvider;
import com.sun.jersey.oauth.server.spi.OAuthToken;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.ws.rs.ext.Provider;

/**
 *
 * This class contains an implementation of the jersey-oauth <code>OAuthProvider</code>.
 * The methods in this class are called by the oauth filter that is used.
 * <br/>
 * In order to use this class, you should
 * <ul>
 * <li>include jersey-oauth in your application
 * <li>use the default oauth-filter in jersey-oauth, or create a custom one
 * and register it via the
 * <code>com.sun.jersey.spi.container.ContainerRequestFilters</code>
 * property.
 * </ul>
 * Since this class is annotated with <code>javax.ws.rs.ext.Provider</code>, the default oauth-filter will
 * use this class to get and generate tokens.
 * @author joeri
 */
@Provider
public class DaliOAuthProvider implements OAuthProvider {
  OAuthBean oauthBean = lookup(OAuthBean.class);

  @Override
  public OAuthConsumer getConsumer(String consumerKey) {
    OAuthConsumer consumer =  oauthBean.findDaliServiceConsumerByConsumerKey(consumerKey);
    return consumer;
  }

  @Override
  public OAuthToken newRequestToken(String consumerKey, String callback,
          Map<String, List<String>> attributes) {
    return oauthBean.createRequestToken(consumerKey, callback, attributes);
  }

  @Override
  public OAuthToken getRequestToken(String requestToken) {
    return oauthBean.findDaliToken(requestToken, DaliToken.Type.OAUTH_REQUEST_TOKEN);
  }

  @Override
  public OAuthToken newAccessToken(OAuthToken oauthRequestToken, String verifier) {
    return oauthBean.createAccessToken(oauthRequestToken.getToken(), verifier);
  }

  @Override
  public OAuthToken getAccessToken(String accessToken) {
    return oauthBean.findDaliToken(accessToken, DaliToken.Type.OAUTH_ACCESS_TOKEN);
  }

  private <T> T lookup(Class<T> clazz) {
    try {
      Context c = new InitialContext();
      return (T) c.lookup("java:module/" + clazz.getSimpleName());
    } catch (NamingException ne) {
      Constants.LOGGER.log(Level.SEVERE, "exception caught", ne);
      throw new RuntimeException(ne);
    }
  }
}
