package com.lodgon.dali.core.oauth.entity;

import com.lodgon.dali.core.oauth.Constants;
import com.sun.jersey.oauth.server.spi.OAuthConsumer;
import java.io.Serializable;
import java.security.Principal;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

/**
 *
 * @author joeri
 */
@Entity
@NamedQueries({
  @NamedQuery(
    name = "DaliServiceConsumer.countAll",
    query = "SELECT COUNT(dsc) FROM DaliServiceConsumer dsc"
  ),
  @NamedQuery(
    name = "DaliServiceConsumer.findAll",
    query = "SELECT dsc FROM DaliServiceConsumer dsc ORDER BY dsc.creationDate DESC"
  ),
  @NamedQuery(
    name = "DaliServiceConsumer.findByConsumerKey",
    query = "SELECT dsc FROM DaliServiceConsumer dsc WHERE dsc.consumerKey = :consumerKey"
  )
})
@XmlRootElement
@XmlAccessorType(XmlAccessType.PROPERTY)
public class DaliServiceConsumer implements OAuthConsumer, Serializable, Principal {
  private static DateFormat w3cformat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");

  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  private int id;
  private long creationDate;
  
  private String name;
  private String description;

  private String callback;
  private String organization;
  private String consumerKey;
  private String consumerSecret;

  @Enumerated(EnumType.ORDINAL)
  private Status status;

  @ElementCollection(fetch = FetchType.EAGER)
  private List<String> roles;

  @XmlElement(name="creationDate")
  private String getFormattedCreationDate() {
    return w3cformat.format(new Date(creationDate));
  }

  private void setFormattedCreationDate(String formattedCreationDate) {
		try {
			this.creationDate = w3cformat.parse(formattedCreationDate).getTime();
		} catch (ParseException ex) {
			Constants.LOGGER.log(Level.SEVERE, null, ex);
			this.creationDate = 0;
		}
  }

	@XmlTransient
  public long getCreationDate() {
    return creationDate;
  }

  public void setCreationDate(long creationDate) {
    this.creationDate = creationDate;
  }

  public String getCallback() {
    return callback;
  }

  public void setCallback(String callback) {
    this.callback = callback;
  }

  public String getConsumerKey() {
    return consumerKey;
  }

  public void setConsumerKey(String consumerKey) {
    this.consumerKey = consumerKey;
  }

  public String getConsumerSecret() {
    return consumerSecret;
  }

  public void setConsumerSecret(String consumerSecret) {
    this.consumerSecret = consumerSecret;
  }

  @XmlElement()
  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public int getId() {
    return id;
  }

  public void setId(int id) {
    this.id = id;
  }

  public String getOrganization() {
    return organization;
  }

  public void setOrganization(String organization) {
    this.organization = organization;
  }

  public List<String> getRoles() {
    return roles;
  }

  public void setRoles(List<String> roles) {
    this.roles = roles;
  }

  public Status getStatus() {
    return status;
  }

  public void setStatus(Status status) {
    this.status = status;
  }

  @Override
  public String getKey() {
    return consumerKey;
  }

  @Override
  public String getSecret() {
    return consumerSecret;
  }

  @Override
  public Principal getPrincipal() {
    return this;
  }

  @Override
  public boolean isInRole(String role) {
    return getRoles().contains(role);
  }

  @Override
  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public enum Status {
    ACTIVE, BLOCKED, REMOVED
  }
}
