package com.lodgon.dali.core.oauth.entity;

import com.sun.jersey.core.util.MultivaluedMapImpl;
import com.sun.jersey.oauth.server.spi.OAuthConsumer;
import com.sun.jersey.oauth.server.spi.OAuthToken;
import java.io.Serializable;
import java.security.Principal;
import java.util.List;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.ws.rs.core.MultivaluedMap;

/**
 *
 * @author joeri
 */
@Entity
@NamedQueries({
  @NamedQuery(
    name =  "DaliToken.findByTokenAndType",
    query = "SELECT dt FROM DaliToken dt WHERE dt.token = :token AND dt.type = :type ORDER BY dt.creationDate DESC"
  ),
  @NamedQuery(
    name =  "DaliToken.findByPrincipal",
    query = "SELECT dt FROM DaliToken dt WHERE dt.oauthUser = :principal AND dt.type = :type ORDER BY dt.creationDate DESC"
  ),
  @NamedQuery(
    name =  "DaliToken.findUnusedByTypeOlderThan",
    query = "SELECT dt FROM DaliToken dt WHERE dt.oauthUser is null AND dt.type = :type and dt.creationDate < :date"
  ),
  @NamedQuery(
    name =  "DaliToken.findByConsumerAndPrincipal",
    query = "SELECT dt FROM DaliToken dt WHERE dt.daliServiceConsumer = :consumer AND dt.oauthUser = :principal AND dt.type = :type ORDER BY dt.creationDate DESC"
  )
})
public class DaliToken implements OAuthToken, Serializable {
  @Id
  private String token;
  private String secret;

  private Type type;

  private long creationDate;
  private String verifier;
  private String callback;

  @ManyToOne
  private DaliServiceConsumer daliServiceConsumer;

  @ManyToOne
  private OAuthUser oauthUser;

  @ElementCollection(fetch = FetchType.EAGER)
  private List<Attribute> attrs;

  @Override
  public String getToken() {
    return token;
  }

  public void setToken(String token) {
    this.token = token;
  }

  @Override
  public String getSecret() {
    return secret;
  }

  public void setSecret(String secret) {
    this.secret = secret;
  }

  @Override
  public OAuthConsumer getConsumer() {
    return daliServiceConsumer;
  }

  public DaliServiceConsumer getDaliServiceConsumer() {
    return daliServiceConsumer;
  }

  public void setDaliServiceConsumer(DaliServiceConsumer daliServiceConsumer) {
    this.daliServiceConsumer = daliServiceConsumer;
  }

  public List<Attribute> getAttrs() {
    return attrs;
  }

  public void setAttrs(List<Attribute> attrs) {
    this.attrs = attrs;
  }

  @Override
  public MultivaluedMap<String, String> getAttributes() {
    MultivaluedMap<String, String> attributes = new MultivaluedMapImpl();
    for (Attribute attr : attrs) {
      attributes.add(attr.getKey(), attr.getValue());
    }
    return attributes;
  }

  @Override
  public Principal getPrincipal() {
    return oauthUser;
  }

  public OAuthUser getOauthUser() {
    return oauthUser;
  }

  public void setOauthUser(OAuthUser oauthUser) {
    this.oauthUser = oauthUser;
  }

  @Override
  public boolean isInRole(String role) {
    return daliServiceConsumer.isInRole(role);
  }

  public long getCreationDate() {
    return creationDate;
  }

  public void setCreationDate(long creationDate) {
    this.creationDate = creationDate;
  }

  public Type getType() {
    return type;
  }

  public void setType(Type type) {
    this.type = type;
  }

  public String getVerifier() {
    return verifier;
  }

  public void setVerifier(String verifier) {
    this.verifier = verifier;
  }

  public String getCallback() {
    return callback;
  }

  public void setCallback(String callback) {
    this.callback = callback;
  }

  public enum Type {
    OAUTH_REQUEST_TOKEN, OAUTH_ACCESS_TOKEN
  }
}
